
   ####################################################################################
   ### 	R program for fitting Bayesian multivariate density deconvolution models  	### 
   ###		(last modified on Aug 03, 2016, Duke University, Durham, NC, USA)		###
   ####################################################################################   
   
   
   	#################
   	### Author(s) ###
   	#################
   
# Author of the codes: Abhra Sarkar (abhra.stat@gmail.com)
# Authors of the paper: Abhra Sarkar, Debdeep Pati, Antik Chakraborty, Bani K. Mallick, Raymond J. Carroll
# Author for correspondence: Raymond J. Carroll (carroll@stat.tamu.edu)

 
   	###################
   	### Description ###
   	###################
   
# The goal is to estimate the density f_{X} of the vector valued variable of interest \bX. 
# Precise observations on \bX are NOT available.
# Instead, replicated proxies contmainated by measurement errors \bU are generated by the model: 
# \bW_{ij} = \bX_{i} + \bU_{ij}, for i=1,2,...,n, and j=1,...,m_{i}.
#
# The density f_{X} is modeled using overfitted finite mxitures of multivariate normal kernels:
# 		f_{X} = \sum_{k=1}^{K} \pi_{k} \MVN_{d}(\bmu_{k},\bSigma_{k}).
#
# The measurement errors \bU are assumed to have the following structure: 
#		\bU = S(\bX)\bepsilon,
# where S(\bX) = \diag(s_{1}(X_{1}),...,s_{d}(X_{d})) models the conditional variability in the measurement errors.
# The scaled errors \bepsilon are assumed to be independent from \bX.
#  
# The density of the scaled errors is also modeled using finite mixtures of multivariate normal kernels: 
# 		f_{\epsilon} = \sum_{k=1}^{K} \pi_{k} \MVN_{d}(\bmu_{k},\bSigma_{k}),
# subject to the moment restriction:
#		\sum_{k=1}^{K} \pi_{k}\bmu_{k} = 0.
# 
# The variance functions s_{m}^{2}(X_{m}) are estimated by fitting reparametrized versions of the univariate submodels. 
# These estmates are then kept fixed while fitting the multivariate model.
#
# Multivariate normal priors are assigned on \bmu_{k}. Inverese-Wishart priors are assigned on \bSigma_{k}.







rm(list=ls())

setwd("/Users/abhrasarkar/Dropbox/Projects with Dr. C/X ME Multivariate Density Deconvolution/Final Codes")
source("BAYES_MVT_DECON_FUNCTS.R")
source("BAYES_UNIV_DECON_HTR.R")

seedno = 1
set.seed(seedno)

library(mvtnorm)
library(mclust)
library(RColorBrewer)
library(MCMCpack)	# Dependencies : coda, lattice and MASS
library(corpcor)



#######################
### MCMC parameters ###
#######################

simsize = 300
burnin = 100



	#####################
	### Load Data Set ###
	#####################


Data = load("Simulated_Data_Set.RData")

d = ncol(ws)

# For real data sets, it is recommended that the surrogates be scaled to be unitless and have common range. 
# This can be done by uncommenting lines 103-105 below.
# Non-linear transformations that do not preserve the additivity of the data generating model are not recommended.  
# The common range of values of X is specified by "range.xs". 
# Most real data sets will be highly right skewed. 
# So setting the range at 20 units may bring the effective range to much smaller values. 
# For the EATS data set analyzed in the main paper, for example, this brings the effective range to about 10 units. 

#range.xs = 20
#for(ii in 1:d)
#	ws[,ii] = range.xs*(ws[,ii]-min(ws[,ii]))/(max(ws[,ii])-min(ws[,ii])) 

mis = as.data.frame(table(inds))$Freq	# Number of replicates per individual
n.xs = n = length(unique(inds))			# Total no of individuals
inds = rep(1:n,times=mis)				# Individual IDs replicated to match the proxies
n.es = N = sum(mis)						# Total no of surrogates

inds1 = inds2 = numeric(n)
inds1[1] = 1
inds2[1] = inds1[1]+mis[1]-1
for(ii in 2:n)
	{
	inds1[ii] = inds1[ii-1]+mis[ii-1]
	inds2[ii] = inds1[ii] + mis[ii]-1
	}
	
xs = matrix(0,nrow=n,ncol=d)
for(ii in 1:n)
	xs[ii,] = colMeans(ws[inds1[ii]:inds2[ii],])
us = (ws - xs[inds,])



###########################################
### Prior Parameters and Initial Values ###
###########################################

inds1 = inds2 = numeric(n)
inds1[1] = 1
inds2[1] = inds1[1]+mis[1]-1
for(ii in 2:n)
	{
	inds1[ii] = inds1[ii-1]+mis[ii-1]
	inds2[ii] = inds1[ii] + mis[ii]-1
	}
	
xs = matrix(0,nrow=n,ncol=d)
K.t = 10
knots = matrix(0,nrow=K.t,ncol=d)
thetas = matrix(0,nrow=K.t+1,ncol=d)
es = matrix(0,nrow=sum(mis),ncol=d)


### Run Univariate Submodels ###
# This produces estimates of the variance functions that capture the conditional variability. 
# This also gives initial estimates of X, number of clusters etc. as important by-products.

simsize_univ = 3000
burnin_univ = 1000
for(jj in 1:d)
{
filename = paste("MVT_Decon_HTR_Initial_Values",jj,sep="_")
filename = paste(filename,".RData",sep="") 
print(filename)
if(FALSE) 
	{
	# Set the switch to TRUE for the first time. For experimentation with subsequent runs, read from saved files.		
	univ_results = UNIV_DECON(ws[,jj],mis,5,5,K.t,simsize_univ,burnin_univ,TRUE) 
	save(file=filename,univ_results)
	}
load(filename)
knots[,jj] = univ_results$knots			# The knot points that define the B-splines
xs[,jj] = univ_results$xs				# The initial estimates of X
thetas[,jj] = univ_results$thetas		# The log cofficients of the B-splines
es[,jj] = univ_results$es				# The initial scaled residuals
}
xs.current = xs.proposed = xs


Var.xs = function(xs,knots,thetas)
	{
	xs = as.matrix(xs)
	knots = as.matrix(knots)
	thetas = as.matrix(thetas)
	y = matrix(0,nrow=nrow(xs),ncol=ncol(xs))
	for(jj in 1:ncol(as.matrix(y)))
		y[,jj] = B.basis(xs[,jj],knots[,jj])%*%exp(thetas[,jj])
	return(y)
	}


# In the following, 
# K denotes the number of mixture components, 
# K.star denotes the number of latent factors,
# Pi denotes the mixture probabilities, C denotes the cluster labels, 
# Mu denotes the cluster mean, Sigma denotes the cluster covariance matrix,
# alpha denotes the hyper-parameter of the Dirichlet prior on the mixture probabilities,
# Mu0 and Sigma0 denote hyper-parameters of multivariate normal priors on Mu,
# nu0 and Omega0 denote hyper-parameters of inverse wishart priors on Sigma.


### For the density of X ###

alpha.xs = 1
Mu0.xs = colMeans(xs)
Sigma0.xs = diag(apply(xs,2,"var"))

d_clust = Mclust(as.matrix(xs), modelNames=c("VVV"), G=1:5)
m.best = length(unique(d_clust$class))
cat("Model-based optimal number of clusters:", m.best, "\n")
K.xs = m.best+3
Pi.xs = rep(0,K.xs)
Pi.xs[1:m.best] = c(d_clust$parameters$pro)
C.xs = d_clust$classification
Mu.xs = matrix(rep(colMeans(xs),K.xs),nrow=K.xs,ncol=d,byrow=T)
Mu.xs[1:m.best,] = t(d_clust$parameters$mean)
Sigma.xs = array(0,dim=c(d,d,K.xs))
for(kk in 1:m.best)
	{
	temp = which(C.xs==kk)
	Sigma.xs[,,kk] = cov(xs[temp,])
	}
for(kk in (m.best+1):K.xs)
	Sigma.xs[,,kk] = cov(xs)

nu0.xs = d+2
Omega0.xs = (nu0.xs-d-1)*cov(xs) 


### For the density of the scaled errors ###

alpha.es = 1
Mu0.es = rep(0,d)
Sigma0.es = diag(apply(es,2,"var"))

d_clust = suppressWarnings(Mclust(as.matrix(es), modelNames=c("VVV"), G=1:5))
m.best = length(unique(d_clust$class))
cat("Model-based optimal number of clusters:", m.best, "\n")
K.es = m.best+3
Pi.es = rep(0,K.es)
Pi.es[1:m.best] = c(d_clust$parameters$pro)
C.es = d_clust$classification
Mu.es = matrix(rep(colMeans(es),K.es),nrow=K.es,ncol=d,byrow=T)
Mu.es[1:m.best,] = t(d_clust$parameters$mean)
Sigma.es = array(0,dim=c(d,d,K.es))
for(kk in 1:m.best)
	{
	temp = which(C.es==kk)
	Sigma.es[,,kk] = cov(es[temp,])
	}
for(kk in (m.best+1):K.es)
	Sigma.es[,,kk] = cov(es)

nu0.es = d+2
Omega0.es = (nu0.es-d-1)*cov(es) 







###########################################
### MCMC Storage and Proposal Variables ###
###########################################

d.ordinates.xs = matrix(0,nrow=n.xs,ncol=K.xs)
d.ordinates.es = matrix(0,nrow=n.es,ncol=K.es)

min.xs = apply(apply(xs,2,"range"),2,"min")
max.xs = apply(apply(xs,2,"range"),2,"max")
Sigma.xs.proposal = diag(0.1*(max.xs-min.xs)/6)

current.likelihood.temp = proposed.likelihood.temp = numeric(N)
prior.current.xs = prior.proposed.xs = rep(1,n)
	
acc.thetas = numeric(d)

S.xs = S.xs.current = S.xs.proposed = matrix(1,nrow=n,ncol=d)
postmean.xs.mat = matrix(0,nrow=n,ncol=d)

xs.MCMC 		= array(0,dim=c(n,d,simsize))					

Pi.xs.MCMC		= array(0,dim=c(simsize-burnin,K.xs))					
Mu.xs.MCMC		= array(0,dim=c(K.xs,d,simsize-burnin))				
Sigma.xs.MCMC 		= array(0,dim=c(d,d,K.xs,simsize-burnin))			

Pi.es.MCMC	 	= array(0,dim=c(simsize-burnin,K.es))	
Mu.es.MCMC 		= array(0,dim=c(K.es,d,simsize-burnin))		
Sigma.es.MCMC 	= array(0,dim=c(d,d,K.es,simsize-burnin))			

K.xs.MCMC 		= numeric(simsize)
K.es.MCMC 		= numeric(simsize)





	##################
	### Start MCMC ###
	##################
	

for(iii in 1:simsize)
	{
	if(iii%%100==0)
		print(iii)
			
			
	### Updating the parameters of the Distribution of X	
		
	### Updating the cluster indices
	for(kk in 1:K.xs)
		d.ordinates.xs[,kk] = dmvnorm(xs,Mu.xs[kk,],Sigma.xs[,,kk])
	for(ii in 1:n.xs)
		C.xs[ii] = sample(K.xs,1,prob=Pi.xs*d.ordinates.xs[ii,])
	
	
	### Updating cluster probabilities
	n.kk.xs = tabulate(C.xs,nbins=K.xs)
	Pi.xs = rdirichlet(1,alpha.xs+n.kk.xs)	
	
	

	### Updating Mu.xs and Sigma.xs
	for(kk in 1:K.xs)
		{
		temp = which(C.xs==kk)
		xs.temp = xs[temp,]
		if(length(temp)==1)
			xs.temp = t(xs.temp)
		Sigma0.xs.temp = solve(solve(Sigma0.xs) + length(temp) * solve(Sigma.xs[,,kk]))
		Mu0.xs.temp = Sigma0.xs.temp %*% (solve(Sigma0.xs) %*% Mu0.xs + solve(Sigma.xs[,,kk]) %*% colSums(xs.temp))
		Mu.xs[kk,] = rmvnorm(1,Mu0.xs.temp,Sigma0.xs.temp)

		MeanMat = matrix(rep(Mu.xs[kk,],length(temp)),ncol=d,byrow=T)
		if(length(temp)>0)
			Sigma.xs[,,kk] = riwish(nu0.xs+length(temp),Omega0.xs  +  t(xs.temp-MeanMat)%*%(xs.temp-MeanMat))
		}
	
		
	
	### Updating X
	
	if(iii>min(50,burnin/2))
	{
	xs.proposed = xs.current + rmvnorm(n,rep(0,d),Sigma.xs.proposal)
	S.xs.current = sqrt(Var.xs(xs.current,knots,thetas))
	S.xs.proposed = sqrt(Var.xs(xs.proposed,knots,thetas))
	S.xs.proposed[S.xs.proposed<0.0000001] = 0.0001

	for(ii in 1:n)
		{
		kk.xs = C.xs[ii]
		prior.current.xs[ii] = dmvnorm(xs.current[ii,],Mu.xs[kk.xs,],Sigma.xs[,,kk.xs])
		prior.proposed.xs[ii] = dmvnorm(xs.proposed[ii,],Mu.xs[kk.xs,],Sigma.xs[,,kk.xs])
			
		for(jj in inds1[ii]:inds2[ii])
			{
			kk.es = C.es[jj]
			
			Covmat.temp.current = diag(S.xs.current[ii,]) %*% Sigma.es[,,kk.es] %*% diag(S.xs.current[ii,])
			current.likelihood.temp[jj] = dmvnorm(ws[jj,],xs.current[ii,]+S.xs.current[ii,]*Mu.es[kk.es,],Covmat.temp.current)
			
			Covmat.temp.proposed = diag(S.xs.proposed[ii,]) %*% Sigma.es[,,kk.es] %*% diag(S.xs.proposed[ii,])
			proposed.likelihood.temp[jj] = dmvnorm(ws[jj,],xs.proposed[ii,]+S.xs.proposed[ii,]*Mu.es[kk.es,],Covmat.temp.proposed)
			}
		}
		
	current.likelihood = tapply(current.likelihood.temp,inds,"prod")
	proposed.likelihood = tapply(proposed.likelihood.temp,inds,"prod")
	
	mh.ratio = (prior.proposed.xs*proposed.likelihood)/(prior.current.xs*current.likelihood)
	mh.ratio[which(is.nan(mh.ratio)==T)] = 0
	acc.prob = runif(n)
	inds.to.replace = (1:n)[acc.prob<mh.ratio]
	xs[inds.to.replace,] = xs.current[inds.to.replace,] = xs.proposed[inds.to.replace,]
	us = (ws-xs[inds,])
	if((iii>min(50,burnin/2))&&(T))
	{
	S.xs = sqrt(Var.xs(xs[inds,],knots,thetas))
	es = us/S.xs}
	}						


	
	### Updating the parameters of the Distribution of measurement errors		
		
	### Updating the cluster indices
	for(kk in 1:K.es)
		d.ordinates.es[,kk] = dmvnorm(es,Mu.es[kk,],Sigma.es[,,kk])
	for(ii in 1:n.es)
		C.es[ii] = sample(K.es,1,prob=Pi.es*d.ordinates.es[ii,])
	
	
	### Updating cluster probabilities
	n.kk.es = tabulate(C.es,nbins=K.es)
	Pi.es = rdirichlet(1,alpha.es+n.kk.es)	

	
	### Updating Mu.es
	if(K.es==1)
		{Mu.es = matrix(0,nrow=1,ncol=d)}else
	{Big.Mu0.es = numeric((K.es+1)*d)
	Big.Sigma0.es = matrix(0,nrow=(K.es+1)*d,ncol=(K.es+1)*d)
	for(kk in 1:K.es)
		{
		temp = which(C.es==kk)
		Sigma0.es.temp = solve(solve(Sigma0.es) + length(temp) * solve(Sigma.es[,,kk]))
		es.temp = es[temp,]						# Takes care of the case n.kk.es[kk]==1
		if(length(temp)==1)
			es.temp = t(es.temp)
		Mu0.es.temp = Sigma0.es.temp %*% (solve(Sigma0.es) %*% Mu0.es + solve(Sigma.es[,,kk]) %*% colSums(es.temp))
		Big.Mu0.es[((kk-1)*d+1):(kk*d)] = Mu0.es.temp
		Big.Sigma0.es[((kk-1)*d+1):(kk*d),((kk-1)*d+1):(kk*d)] = Sigma0.es.temp
	
		Big.Mu0.es[(K.es*d+1):((K.es+1)*d)] = Big.Mu0.es[(K.es*d+1):((K.es+1)*d)] + Pi.es[kk] * Mu0.es.temp
		Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),(K.es*d+1):((K.es+1)*d)] = Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),(K.es*d+1):((K.es+1)*d)] + Pi.es[kk]^2 * Sigma0.es.temp
		Big.Sigma0.es[((kk-1)*d+1):(kk*d),(K.es*d+1):((K.es+1)*d)] = Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),((kk-1)*d+1):(kk*d)] = Pi.es[kk] * Sigma0.es.temp
		}
		
	Big.Mu0.es1 = Big.Mu0.es[1:(K.es*d)] - Big.Sigma0.es[1:(K.es*d),(K.es*d+1):((K.es+1)*d)] %*% solve(Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),(K.es*d+1):((K.es+1)*d)]) %*% Big.Mu0.es[(K.es*d+1):((K.es+1)*d)] 
	Big.Sigma0.es11 = Big.Sigma0.es[1:(K.es*d),1:(K.es*d)] - Big.Sigma0.es[1:(K.es*d),(K.es*d+1):((K.es+1)*d)] %*% solve(Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),(K.es*d+1):((K.es+1)*d)]) %*% Big.Sigma0.es[(K.es*d+1):((K.es+1)*d),1:(K.es*d)]

	Mu.es[1:(K.es-1),] = matrix(rmvnorm(1,Big.Mu0.es1[1:((K.es-1)*d)],Big.Sigma0.es11[1:((K.es-1)*d),1:((K.es-1)*d)]),nrow=(K.es-1),byrow=T)
	if(K.es>2)
		{Mu.es[K.es,] = - colSums(Pi.es[1:(K.es-1)]* Mu.es[1:(K.es-1),]) / Pi.es[K.es]}else
		{Mu.es[K.es,] = - Pi.es[1:(K.es-1)]* Mu.es[1:(K.es-1),] / Pi.es[K.es]}
	}
	
	
	
	### Updating Sigma.es
	for(kk in 1:K.es)
		{
		temp = which(C.es==kk)
		es.temp = es[temp,]
		if(length(temp)==1)
			es.temp = t(es.temp)
		MeanMat = matrix(rep(Mu.es[kk,],length(temp)),ncol=d,byrow=T)
		if(length(temp)>0)
			Sigma.es[,,kk] = riwish(nu0.es+length(temp),Omega0.es+t(es.temp-MeanMat)%*%(es.temp-MeanMat))
		}
		
	
			
							
	# Store the MCMC Output			
	xs.MCMC[,,iii] = xs					
	K.xs.MCMC[iii] = length(which(Pi.xs>0.05))
	K.es.MCMC[iii] = length(which(Pi.es>0.05))

	if(iii>burnin)
	{
	Pi.xs.MCMC[iii-burnin,] 	= Pi.xs					
	Mu.xs.MCMC[,,iii-burnin] 	= Mu.xs				
	Sigma.xs.MCMC[,,,iii-burnin] 	= Sigma.xs		

	Pi.es.MCMC[iii-burnin,] 	= Pi.es	
	Mu.es.MCMC[,,iii-burnin] 	= Mu.es				
	Sigma.es.MCMC[,,,iii-burnin]	= Sigma.es		
	}
	
	}


xs.est = apply(xs.MCMC[,,(burnin+1):simsize],c(1,2),mean)
us.est = (ws-xs.est[inds,])
S.xs.est = sqrt(Var.xs(xs.est[inds,],knots,thetas))
es.est = us.est/S.xs.est







	######################
	### Create Figures ###
	######################

xs.grid.length = 100
xs.grid.lim.lwr = min(apply(Mu.xs,2,min))-3
xs.grid.lim.upr = max(apply(Mu.xs,2,max))+3
xs.grid = seq(xs.grid.lim.lwr,xs.grid.lim.upr,len=xs.grid.length)
xs.grid.mat = cbind(rep(xs.grid,times=rep(xs.grid.length,xs.grid.length)),rep(xs.grid,times=xs.grid.length))
density.xs.est.univ = matrix(0,nrow=xs.grid.length,ncol=d)
density.xs.est.array = array(0,dim=c(d,d,xs.grid.length,xs.grid.length))
sum_set = (simsize-burnin-min(25,simsize-burnin)+1):(simsize-burnin)

 
for(mm in 1:d)
	{
	for(kk in 1:K.xs)
		for(iii in sum_set)
			{
			CovMat = Sigma.xs.MCMC[,,kk,iii]
			density.xs.est.univ[,mm] = density.xs.est.univ[,mm] + Pi.xs.MCMC[iii,kk] * dnorm(xs.grid,Mu.xs.MCMC[kk,mm,iii],sqrt(CovMat[mm,mm]))
			}
	density.xs.est.univ[,mm] = density.xs.est.univ[,mm]/length(sum_set)
	}
	
indices = combn(d,2)
for(ii in 1:ncol(indices))
	{
	dim.indices = indices[,ii]
	d1 = dim.indices[1]
	d2 = dim.indices[2]
			
	density.xs.est.vect = numeric(xs.grid.length^2)
	for(kk in 1:K.xs)
		for(iii in sum_set)
			{
			CovMat = Sigma.xs.MCMC[,,kk,iii]
			density.xs.est.vect = density.xs.est.vect + Pi.xs.MCMC[iii,kk] * dmvnorm(xs.grid.mat,Mu.xs.MCMC[kk,c(d1,d2),iii],CovMat[c(d1,d2),c(d1,d2)])
			}
	density.xs.est.vect = density.xs.est.vect/length(sum_set)
	density.xs.est.array[d1,d2,,] = matrix(density.xs.est.vect,nrow=xs.grid.length,byrow=T) 
	}




es.grid.length = 100
es.grid.lim.lwr = -4
es.grid.lim.upr = 4
es.grid = seq(es.grid.lim.lwr,es.grid.lim.upr,len=es.grid.length)
es.grid.delta = (es.grid[2]-es.grid[1])
es.grid.mat = cbind(rep(es.grid,times=rep(es.grid.length,es.grid.length)),rep(es.grid,times=es.grid.length))
density.es.est.univ = matrix(0,nrow=es.grid.length,ncol=d)
density.es.est.array = array(0,dim=c(d,d,es.grid.length,es.grid.length))

	
for(mm in 1:d)
	{
	for(kk in 1:K.es)
		for(iii in sum_set)
			{
			CovMat = Sigma.es.MCMC[,,kk,iii] 
			density.es.est.univ[,mm] = density.es.est.univ[,mm] + Pi.es.MCMC[iii,kk] * dnorm(es.grid,Mu.es.MCMC[kk,mm,iii],sqrt(CovMat[mm,mm]))
			}
	density.es.est.univ[,mm] = density.es.est.univ[,mm]/length(sum_set)
	}

indices = combn(d,2)
for(ii in 1:ncol(indices))
	{
	dim.indices = indices[,ii]
	d1 = dim.indices[1]
	d2 = dim.indices[2]
		
	density.es.est.vect = numeric(es.grid.length^2)
	for(kk in 1:K.es)
		for(iii in sum_set)
			{
			CovMat = Sigma.es.MCMC[,,kk,iii]
			density.es.est.vect = density.es.est.vect + Pi.es.MCMC[iii,kk] * dmvnorm(es.grid.mat,Mu.es.MCMC[kk,c(d1,d2),iii],CovMat[c(d1,d2),c(d1,d2)])
			}				
	density.es.est.vect = density.es.est.vect/length(sum_set)
	density.es.est.array[d1,d2,,] = matrix(density.es.est.vect,nrow=es.grid.length,byrow=T) 
	}







pdf("MVT_DECON_HTR_MIW.pdf")

#######################
### 	XS	& ES	### 	Marginals of XS
#######################

laymat = diag(1:d)
laymat[lower.tri(laymat)] =  (d+1):(d*(d+1)/2)
laymat = t(laymat) 										# fixes the upper part
laymat_temp = diag(rep(0,d))
laymat_temp[upper.tri(laymat_temp)] = ((d*(d+1)/2)+1):d^2
laymat = laymat+t(laymat_temp)								# fixes the lower part
laymat

layout(laymat) 

par(mar=rep(1.85,4))
xs = data.frame(xs)	
my.cols = rev(brewer.pal(11, "RdYlBu"))
for(i in 1:d) 
	{
	plot(xs.grid,density.xs.est.univ[,i],col="blue",type="l",lwd=2)
	}
for(i in 1:(d-1))
	for(j in (i+1):d)
		{
		image(xs.grid,xs.grid,density.xs.est.array[i,j,,],col=terrain.colors(100))
		contour(xs.grid,xs.grid,density.xs.est.array[i,j,,],nlevels=45,drawlabels=FALSE,col=my.cols[6:11],add=T)
		legend("bottomright",legend=paste(i,j),cex=1,bty="n")
		} 
for(i in 2:d)
	for(j in 1:(i-1))
		{
		image(es.grid,es.grid,density.es.est.array[j,i,,],col=terrain.colors(100))
		contour(es.grid,es.grid,density.es.est.array[j,i,,],nlevels=45,drawlabels=FALSE,col=my.cols[6:11],add=T)
		legend("bottomright",legend=paste(j,i),cex=1,bty="n")
		} 



#######################
### 	XS	& ES	### 	Marginals of ES
#######################

laymat = diag(1:d)
laymat[lower.tri(laymat)] =  (d+1):(d*(d+1)/2)
laymat = t(laymat) 										# fixes the upper part
laymat_temp = diag(rep(0,d))
laymat_temp[upper.tri(laymat_temp)] = ((d*(d+1)/2)+1):d^2
laymat = laymat+t(laymat_temp)								# fixes the lower part
laymat

layout(laymat) 

par(mar=rep(1.85,4))
xs = data.frame(xs)	
my.cols = rev(brewer.pal(11, "RdYlBu"))
for(i in 1:d) 
	{
	plot(es.grid,density.es.est.univ[,i],col="blue",type="l",lwd=2)
	}
for(i in 1:(d-1))
	for(j in (i+1):d)
		{
		image(xs.grid,xs.grid,density.xs.est.array[i,j,,],col=terrain.colors(100))
		contour(xs.grid,xs.grid,density.xs.est.array[i,j,,],nlevels=45,drawlabels=FALSE,col=my.cols[6:11],add=T)
		legend("bottomright",legend=paste(i,j),cex=1,bty="n")
		} 
for(i in 2:d)
	for(j in 1:(i-1))
		{
		image(es.grid,es.grid,density.es.est.array[j,i,,],col=terrain.colors(100))
		contour(es.grid,es.grid,density.es.est.array[j,i,,],nlevels=45,drawlabels=FALSE,col=my.cols[6:11],add=T)
		legend("bottomright",legend=paste(j,i),cex=1,bty="n")
		} 


#################################
### 	VARIANCE FUNCTIONS    ###
#################################

par(mfrow=c(2,2))
par(mar=c(rep(2,4)))
for(jj in 1:d)
	{	
	xlimlwr = max(min(xs[,jj]),xs.grid[1])
	xlimupr = min(max(xs[,jj]),xs.grid[90])	
	s2is = as.vector(tapply(ws[,jj],inds,var))
	plot(xs[,jj],s2is,xlim=c(xlimlwr,xlimupr),ylim=c(0,30),xlab="X",ylab="Var Function",pch=16,col="gray60",cex=0.5)
	points(xs.grid,B.basis(xs.grid,knots[,jj])%*%exp(thetas[,jj]),type="l",lty=1,col="blue",lwd=2)
	}
par(mfrow=c(1,1))
dev.off()








###########################
### 	SAVE RESULTS    ###
###########################

save(ws, inds, mis, d, n, N, simsize_univ, burnin_univ, knots, thetas,
	xs.MCMC, K.xs.MCMC,	K.es.MCMC, 
	Pi.xs.MCMC, Mu.xs.MCMC,	Sigma.xs.MCMC,
	Pi.es.MCMC,	Mu.es.MCMC,	Sigma.xs.MCMC,
	sum_set,
	xs.grid, density.xs.est.univ, density.xs.est.array,
	es.grid, density.es.est.univ, density.es.est.array, file="MVT_DECON_HTR_MIW.RData")



