
   ####################################################################################
   ### 	R program for fitting Bayesian multivariate density deconvolution models   	### 
   ###			Fits a naive Inv Wishart Model that ignores measurement errors	   	###
   ###		(last modified on Aug 03, 2016, Duke University, Durham, NC, USA)		###
   ####################################################################################   
   
   
   	#################
   	### Author(s) ###
   	#################
   
# Author of the codes: Abhra Sarkar (abhra.stat@gmail.com)
# Authors of the paper: Abhra Sarkar, Debdeep Pati, Antik Chakraborty, Bani K. Mallick, Raymond J. Carroll
# Author for correspondence: Raymond J. Carroll (carroll@stat.tamu.edu)

   	###################
   	### Description ###
   	###################
   
# The goal is to estimate the density f_{X} of the vector valued variable of interest \bX. 
# A Naive model is fitted where measurement errors are ignored. 
# The density f_{X} is modeled using overfitted finite mxitures of multivariate normal kernels:
# 		f_{X} = \sum_{k=1}^{K} \pi_{k} \MVN_{d}(\bmu_{k},\bSigma_{k}).
#
# Multivariate normal priors are assigned on \bmu_{k}. Inverese-Wishart priors are assigned on \bSigma_{k}.


rm(list=ls())

#setwd("/Users/abhrasarkar/Dropbox/Projects with Dr. C/X ME Multivariate Density Deconvolution/Final Codes")
setwd("C:\\Users\\Carroll\\Documents\\My_Documents\\Tex\\papers\\Abhra_Sarkar\\Multivariate_Density_Deconvolution\\Final\\Revision for JASA - 2.1\\Software")
source("BAYES_MVT_DECON_FUNCTS.R")

seedno = 1
set.seed(seedno)

library(mvtnorm)
library(sn)
library(RColorBrewer)
library(MCMCpack)	# Dependencies : coda, lattice and MASS
library(corpcor)



		###########################
		### Set MCMC Parameters ###
		###########################
		
simsize = 300
burnin = 100


	#####################
	### Load Data Set ###
	#####################


Data = load("Simulated_Data_Set.RData")

d = ncol(ws)

# For real data sets, it is recommended that the surrogates be scaled to be unitless and have common range. 
# This can be done by uncommenting lines 103-105 below.
# Non-linear transformations that do not preserve the additivity of the data generating model are not recommended.  
# The common range of values of X is specified by "range.xs". 
# Most real data sets will be highly right skewed. 
# So setting the range at 20 units may bring the effective range to much smaller values. 
# For the EATS data set analyzed in the main paper, for example, this brings the effective range to about 10 units. 

#range.xs = 20
#for(ii in 1:d)
#	ws[,ii] = range.xs*(ws[,ii]-min(ws[,ii]))/(max(ws[,ii])-min(ws[,ii])) 

mis = as.data.frame(table(inds))$Freq	# Number of replicates per individual
n.xs = n = length(unique(inds))			# Total no of individuals
inds = rep(1:n,times=mis)				# Individual IDs replicated to match the proxies
n.es = N = sum(mis)						# Total no of surrogates

inds1 = inds2 = numeric(n)
inds1[1] = 1
inds2[1] = inds1[1]+mis[1]-1
for(ii in 2:n)
	{
	inds1[ii] = inds1[ii-1]+mis[ii-1]
	inds2[ii] = inds1[ii] + mis[ii]-1
	}
	
xs = matrix(0,nrow=n,ncol=d)
for(ii in 1:n)
	xs[ii,] = colMeans(ws[inds1[ii]:inds2[ii],])



###########################################
### Prior Parameters and Initial Values ###
###########################################

inds1 = inds2 = numeric(n)
inds1[1] = 1
inds2[1] = inds1[1]+mis[1]-1
for(ii in 2:n)
	{
	inds1[ii] = inds1[ii-1]+mis[ii-1]
	inds2[ii] = inds1[ii] + mis[ii]-1
	}
	
xs = matrix(0,nrow=n,ncol=d)
for(ii in 1:n)
	{
	xs[ii,] = colMeans(ws[inds1[ii]:inds2[ii],])
	}

# In the following, 
# K denotes the number of mixture components, 
# K.star denotes the number of latent factors,
# Pi denotes the mixture probabilities, C denotes the cluster labels, 
# Mu denotes the cluster mean, Sigma denotes the cluster covariance matrix,
# alpha denotes the hyper-parameter of the Dirichlet prior on the mixture probabilities,
# Mu0 and Sigma0 denote hyper-parameters of multivariate normal priors on Mu,
# nu0 and Omega0 denote hyper-parameters of inverse wishart priors on Sigma.

K.star.xs = (d+1)%/%2
K.xs = 5
clusters.xs = kmeans(xs,K.xs,nstart=10)
Mu.xs = clusters.xs$center
C.xs = clusters.xs$cluster
Pi.xs = tabulate(C.xs)/n

alpha.xs = 1

nu0.xs = d+2
Omega0.xs = (nu0.xs-d-1)*cov(xs) 
Sigma.xs = array(0,dim=c(d,d,K.xs))
for(kk in 1:K.xs)
	{
	temp = which(C.xs==kk)
	if(length(temp)>1)
		Sigma.xs[,,kk] = make.positive.definite(cov(xs[temp,]))
	else
		Sigma.xs[,,kk] = make.positive.definite(cov(xs))		
	}

Mu0.xs = colMeans(xs)
Sigma0.xs = cov(xs)








###########################################
### MCMC Storage and Proposal Variables ###
###########################################

d.ordinates.xs = matrix(0,nrow=n,ncol=K.xs)

min.xs = apply(apply(xs,2,"range"),2,"min")
max.xs = apply(apply(xs,2,"range"),2,"max")

Pi.xs.MCMC		= array(0,dim=c(simsize-burnin,K.xs))
Mu.xs.MCMC		= array(0,dim=c(K.xs,d,simsize-burnin))
Sigma.xs.MCMC 	= array(0,dim=c(d,d,K.xs,simsize-burnin))
K.xs.MCMC 		= numeric(simsize)




	##################
	### Start MCMC ###
	##################
	

for(iii in 1:simsize)
	{
	if(iii%%100==0)
		print(iii)

		
	### Updating the cluster indices
	for(kk in 1:K.xs)
		d.ordinates.xs[,kk] = dmvnorm(xs,Mu.xs[kk,],Sigma.xs[,,kk])
	for(ii in 1:n)
		C.xs[ii] = sample(K.xs,1,prob=Pi.xs*d.ordinates.xs[ii,])
	
	
	### Updating cluster probabilities
	n.kk.xs = tabulate(C.xs,nbins=K.xs)
	Pi.xs = rdirichlet(1,alpha.xs/K.xs+n.kk.xs)	


	### Updating Mu.xs and Sigma.xs
	for(kk in 1:K.xs)
		{
		temp = which(C.xs==kk)
		xs.temp = xs[temp,]
		if(length(temp)==1)
			xs.temp = t(xs.temp)
		Sigma0.xs.temp = solve(solve(Sigma0.xs) + length(temp) * solve(Sigma.xs[,,kk]))
		Sigma0.xs.temp = (Sigma0.xs.temp + t(Sigma0.xs.temp))/2
		Mu0.xs.temp = Sigma0.xs.temp %*% (solve(Sigma0.xs) %*% Mu0.xs + solve(Sigma.xs[,,kk]) %*% colSums(xs.temp))
		Mu.xs[kk,] = rmvnorm(1,Mu0.xs.temp,Sigma0.xs.temp)

		MeanMat = matrix(rep(Mu.xs[kk,],length(temp)),ncol=d,byrow=T)
		if(length(temp)>0)
			{Sigma.xs[,,kk] = riwish(nu0.xs+length(temp),Omega0.xs  +  t(xs.temp-MeanMat)%*%(xs.temp-MeanMat))}else
			{Sigma.xs[,,kk] = riwish(nu0.xs,Omega0.xs)}
		}				
							
						
	# Store the MCMC Output			

	K.xs.MCMC[iii] = length(which(Pi.xs>0.05))
	
	if(iii>burnin)
	{
	Pi.xs.MCMC[iii-burnin,] 	= Pi.xs					
	Mu.xs.MCMC[,,iii-burnin] 	= Mu.xs				
	Sigma.xs.MCMC[,,,iii-burnin] = Sigma.xs		
	}
	
	}


xs.est = xs



	######################
	### Create Figures ###
	######################

xs.grid.length = 100
xs.grid.lim.lwr = min(apply(Mu.xs,2,min))-3
xs.grid.lim.upr = max(apply(Mu.xs,2,max))+3
xs.grid = seq(xs.grid.lim.lwr,xs.grid.lim.upr,len=xs.grid.length)
xs.grid.mat = cbind(rep(xs.grid,times=rep(xs.grid.length,xs.grid.length)),rep(xs.grid,times=xs.grid.length))
density.xs.est.univ = matrix(0,nrow=xs.grid.length,ncol=d)
density.xs.true.array = density.xs.est.array = array(0,dim=c(d,d,xs.grid.length,xs.grid.length))
sum_set = (simsize-burnin-min(25,simsize-burnin)+1):(simsize-burnin)

 
for(mm in 1:d)
	{
	for(kk in 1:K.xs)
		for(iii in sum_set)
			{
			CovMat = Sigma.xs.MCMC[,,kk,iii]
			density.xs.est.univ[,mm] = density.xs.est.univ[,mm] + Pi.xs.MCMC[iii,kk] * dnorm(xs.grid,Mu.xs.MCMC[kk,mm,iii],sqrt(CovMat[mm,mm]))
			}
	density.xs.est.univ[,mm] = density.xs.est.univ[,mm]/length(sum_set)
	}
	
indices = combn(d,2)
for(ii in 1:ncol(indices))
	{
	dim.indices = indices[,ii]
	d1 = dim.indices[1]
	d2 = dim.indices[2]
			
	density.xs.est.vect = numeric(xs.grid.length^2)
	for(kk in 1:K.xs)
		for(iii in sum_set)
			{
			CovMat = Sigma.xs.MCMC[,,kk,iii]
			density.xs.est.vect = density.xs.est.vect + Pi.xs.MCMC[iii,kk] * dmvnorm(xs.grid.mat,Mu.xs.MCMC[kk,c(d1,d2),iii],CovMat[c(d1,d2),c(d1,d2)])
			}
	density.xs.est.vect = density.xs.est.vect/length(sum_set)
	density.xs.est.array[d1,d2,,] = matrix(density.xs.est.vect,nrow=xs.grid.length,byrow=T) 
	}



######################
### Plot Figures ###
######################


pdf("MVT_NAIVE.pdf")

d.star = min(d,4)

###################
### 	XS		###
###################

laymat = diag(1:d.star)
laymat[lower.tri(laymat)] =  (d.star+1):(d.star*(d.star+1)/2)
laymat = t(laymat) 										# fixes the upper part
laymat_temp = diag(rep(0,d.star))
laymat_temp[upper.tri(laymat_temp)] = ((d.star*(d.star+1)/2)+1):d.star^2
laymat = laymat+t(laymat_temp)								# fixes the lower part
laymat

layout(laymat) 

par(mar=rep(1.85,4))
xs = data.frame(xs)	
my.cols = rev(brewer.pal(11, "RdYlBu"))
for(i in 1:d.star) 
	{
	plot(xs.grid,density.xs.est.univ[,i],col="blue",type="l",lwd=2)
	}
for(i in 1:(d.star-1))
	for(j in (i+1):d.star)
		{
		image(xs.grid,xs.grid,density.xs.est.array[i,j,,],col=terrain.colors(100))
		contour(xs.grid,xs.grid,density.xs.est.array[i,j,,],nlevels=45,drawlabels=FALSE,col=my.cols[6:11],add=T)
		legend("bottomright",legend=paste(i,j),cex=1,bty="n")
		} 
dev.off()


